#  nestAbund_read_input.r

read_input_file <- function(fName=NULL) {
#'   reads input file (comma-separated values)
#'
#'@param fname A comma-separated value (.csv) formatted file containing input
#'   data.  See sample_data2.csv for variable names.  Variable names must
#'   be on the 1st line of the file and match the case of these:
#'@details
#'      Peron, G., J. Walker, J. Rotella and J.D. Nichols.
#'      Estimating nest abundance while accounting for time-to-event processes
#'      and imperfect detection.
#'@author Guillaume Peron and James Hines
#'@seealso \code{\link{nestAbund.runmod}}, \code{\link{nestAbund.sim}}, code{\link{nestAbund.gui}}
#'@examples
#'    \dontrun{
#'     read_input_file('sample_data2.csv') #
#'    }
#'
  if (is.null(fName)) {
    fileName<<-tclvalue(tkgetOpenFile())
    if (!nchar(fileName))
      tkmessageBox(title="nestAbund",message = "No file was selected!")
  } else fileName<<-fName
  fileName<<-gsub("\\","/",fileName,fixed=T)
  a=unlist(strsplit(gsub("\\","/",fileName,fixed=T),"/")); l=length(a);
  if (l>1) wd<<-paste(a[1:(l-1)],collapse='/')

  Species<<-NT<<-NULL
        cat('reading fileName=',fileName,'\n')
  xdata<<-read.table(fileName,sep=",",header=T)
  C1<<-dim(xdata)[1]  # Number of nests found
  #######################################################
  ### Define time frames
  #######################################################
  Site<<-xdata$Site; NS<<-length(table(Site)); #NS=num sites
  CC<<-table(xdata$Site)
  v=table(Site,xdata$FirstFound)
  #v=table(rep(Site,2),c(xdata$FirstFound,xdata$LastPresent));
  VisitDays <<-as.numeric(dimnames(v)[[2]])
  TTT <<- length(VisitDays)
  is.visited <<- v>0
                                                 # NT=num time occasions
  for (i in 1:NS) { w=which(v[i,]>0); l=length(w); NT<<-c(NT,l)}
  OccSampl<<-DaySampl<<-matrix(0,NS,max(NT))
  for (i in 1:NS) {
    w=which(v[i,]>0); l=length(w)
    OccSampl[i,1:l]<<-w                       # OccSampl=occasions per site
    DaySampl[i,1:l]<<-as.numeric(names(w))    # DaySampl=Day of occasions
  }
  daysampl1<<-VisitDays #  vector of occasion days
  daymax<<-max(daysampl1) #  last day
  Ttot<<-max(OccSampl) # Total number of occasions
  #######################################################
  ### Format data
  #######################################################
  ### Compute critical dates
  DayStartIncub <<- xdata$FirstFound - xdata$AgeFound + xdata$Max.Clutch.Size +1 ## GP290713
  Hk <<- DayStartIncub + 24
  indexFailBeforeIncub = DayStartIncub > xdata$LastPresent
  DayStartIncub[indexFailBeforeIncub] <<- 99
  Hk[indexFailBeforeIncub] <<- 99
  ### Compute ciritcal occasions
  psik <<- locc <<- bk <<- ek <<- fk <<- rep(99,C1) ;
  for (k in 1:C1) {
    #  last occasion before nest is in incubation
    psik[k] <<- lastocc(DayStartIncub[k],Site[k])
    #  last occasion before nest disappearance (failure or success) is recorded
    locc[k] <<- lastocc(xdata$LastPresent[k],Site[k])
    #  first occasion after initiation (ie nest is available)
    bk[k] <<- birthocc(xdata$FirstFound[k] - xdata$AgeFound[k]+1,Site[k])
    #  entry date
    ek[k] <<- xdata$FirstFound[k] - xdata$AgeFound[k]
    #  occasion first found
    fk[k] <<- sum((1:NT[Site[k]]) * (DaySampl[Site[k],1:NT[Site[k]]]==xdata$FirstFound[k]))
  }
  xxxstd= daysampl1[1:(length(daysampl1)-1)]
  xxxstd = (xxxstd-mean(xxxstd))/sd(xxxstd)
  lastocc_array=matrix(0,daymax+INCUBLEN-DD+1,NS)
  for (i in 1:(daymax+INCUBLEN-DD+1))
    for (j in 1:NS) lastocc_array[i,j]=lastocc(i-1+DD,j)
  #detach(xdata)
  return(xxxstd)
}

summarize_data <- function() {

  FirstFound<<-xdata$FirstFound; LastPresent<<-xdata$LastPresent; Fate<<-xdata$Fate
  Site<<-xdata$Site; AgeFound<<-xdata$AgeFound; AgeDay1<<-xdata$AgeDay1;
  Max.Clutch.Size<<-xdata$Max.Clutch.Size

  sout=c(paste('Number of nests:',C1),
         paste('Nest summary by species and site:'))
  cat(sout,'\n'); species=xdata$Species
  if (length(species)<1) {
    species=NULL
    s=colnames(xdata); spcols=grep('^[A-Z][A-Z][A-Z][A-Z]$',s)
    for (i in 1:dim(xdata)[1]) {
      j=which(xdata[i,spcols]>0);
      species[i]=ifelse(length(j)>0,s[spcols[j]],"????")
    }
  }
  w=table(species,Site); w=cbind(w,rowSums(w)); w=rbind(w,colSums(w))
  colnames(w)[NS+1]="total"; rownames(w)[dim(w)[1]]="total"
  w=as.table(w); dimnames(w)=list(species=rownames(w),Site=colnames(w))
  sout=c(sout,capture.output(print(w)))
  sout=c(sout,' ',paste('Nests initiated',round(mean(AgeFound),1),
          '(sd=',round(sd(AgeFound),1),') days before detection.'))
  sout=c(sout,paste('Nests initiated',-round(mean(AgeDay1),1),
          '(sd=',round(sd(AgeDay1),1),') days after 1st day of study.'))
  sink('nestAbund_input_summary.txt',split=T)
  cat('Nest_abund\n\nFolder:',getwd(),'\n')
  cat('Date  :',date(),'\n\n')
  cat(sout,sep='\n'); sink()
  return(sout)
}
